import 'package:flutter_test/flutter_test.dart';
import 'package:fst_test/fst_test.dart';

void main() {
  group('FoldLeaf', () {
    testWidgets('folds values', (tester) async {
      var (value, write) = await tester.pumpStateTree((context, t) {
        final (v, setV, _) = t.state(() => 0);

        final value = t.fold(0, v, (a, b) => a + b);
        return (value, setV);
      });

      expect(value, equals(0));

      write(1);
      (value, _) = await tester.pumpStateTree();
      expect(value, equals(1));

      write(2);
      (value, _) = await tester.pumpStateTree();
      expect(value, equals(3));

      write(3);
      (value, _) = await tester.pumpStateTree();
      expect(value, equals(6));
    });

    testWidgets(
      'resets to initial value when dependency changes',
      (tester) async {
        var (value, write, reset) = await tester.pumpStateTree((context, t) {
          final (v, setV, _) = t.state(() => 0);
          final (d, setD, _) = t.state(() => Object());

          final value = t.fold(0, v, (a, b) => a + b, (d,));
          return (value, setV, () => setD(Object()));
        });

        expect(value, equals(0));

        write(1);
        (value, _, _) = await tester.pumpStateTree();
        expect(value, equals(1));

        write(2);
        (value, _, _) = await tester.pumpStateTree();
        expect(value, equals(3));

        write(3);
        (value, _, _) = await tester.pumpStateTree();
        expect(value, equals(6));

        write(4);
        (value, _, _) = await tester.pumpStateTree();
        expect(value, equals(10));

        reset();
        (value, _, _) = await tester.pumpStateTree();
        expect(value, equals(0));
      },
    );
  });
}
