import 'package:flutter_test/flutter_test.dart';
import 'package:fst_test/fst_test.dart';

void main() {
  group('StateLeaf', () {
    testWidgets('provides a value and setState', (tester) async {
      var (value, setValue) = await tester.pumpStateTree((context, t) {
        final (v, setV, _) = t.state(() => 42);
        return (v, setV);
      });

      expect(value, 42);
      setValue(43);

      (value, _) = await tester.pumpStateTree();

      expect(value, 43);
    });

    testWidgets('resets value when dependency is changed', (tester) async {
      var (value, setValue, reset) = await tester.pumpStateTree(
        (context, t) {
          final (d, setD, _) = t.state(() => 0);
          final (v, setV, _) = t.state(() => 42, (d,));

          return (v, setV, () => setD(d + 1));
        },
      );

      expect(value, 42);

      setValue(15);
      (value, _, _) = await tester.pumpStateTree();
      expect(value, 15);

      reset();

      (value, _, _) = await tester.pumpStateTree();
      expect(value, 42);
    });
  });
}
